/*

   	 $Id: printjob.h,v 1.1 1998/07/05 22:41:49 boloni Exp $	

	 Requires the Qt and KDE widget libraries, available at no cost at
	 http://www.troll.no and http://www.kde.org respectively
	 
	 Copyright (C) 1997, 1998 
	 Preston Brown (preston.brown@yale.edu)
	 Fester Zigterman (F.J.F.ZigtermanRustenburg@student.utwente.nl)
	 Ian Dawes (iadawes@globalserve.net)
     Neil Hart (nhart@magma.ca)
	 
	 This program is free software; you can redistribute it and/or modify
	 it under the terms of the GNU General Public License as published by
	 the Free Software Foundation; either version 2 of the License, or
	 (at your option) any later version.
	 
	 This program is distributed in the hope that it will be useful,
	 but WITHOUT ANY WARRANTY; without even the implied warranty of
	 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
	 GNU General Public License for more details.
	 
	 You should have received a copy of the GNU General Public License
	 along with this program; if not, write to the Free Software
	 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _PRINTJOB_H
#define _PRINTJOB_H

#include <qobject.h>
#include <qpainter.h>
#include <qprinter.h>
#include <qdatetm.h>
#include <qwidget.h>
#include <qlist.h>

#include "printobject.h"
#include "calobject.h"

/**
 *
 * This is the top-level abstract class of print jobs. The subclasses 
 *  represent various permutations of printing list, day or month views in
 *  various layouts on various pieces of paper.
 *
 * @short top-level abstract class of print jobs
 * @author Neil Hart
 * @version $Revision: 1.1 $
 */
class KDPPrintJob
{
public:
	/**
	 *
	 * Constructs a new print job
	 *
	 * @param printer is the QPrinter we're printing on
	 */
	KDPPrintJob( QPrinter* printer )
	{
		prt = printer;
		prtPainter = new QPainter;
		if ( !(prtPainter->begin( prt )) )
		{
			delete prtPainter;
			prtPainter = NULL;
		}
	}
	virtual ~KDPPrintJob( )
	{
		if ( prtPainter != NULL )
		{
			prtPainter->end();
			delete prtPainter;
		}
	}

	/**
	 *
	 * Initialises the print job, by determining how many pages it will take
	 *  to print the desired view.
	 *
	 * @return returns false if we run into problems, true if everything's OK
	 */
	virtual bool init( ) = 0;

	/**
	 *
	 * Prints the data constructed in init()
	 *
	 * @return returns false if we run into problems, true if everything's OK
	 * @param parent is the top-level widget, so we can construct a progress bar
	 */
	virtual bool print( QWidget* parent ) = 0;

protected:

	QPrinter* prt;
	QPainter* prtPainter;

};


/**
 *
 * This is the abstract top-level class of classes which print their data
 *  on a full printer page.
 *
 * @short prints data on a full printer page
 * @author Neil Hart
 * @version $Revision: 1.1 $
 */
class KDPPrintJobFull : public KDPPrintJob
{
public:
	/**
	 *
	 * Constructs a new print job
	 *
	 * @param printer is the QPrinter we're printing on
	 * @param calendar is the CalObject from which we'll get event info
	 * @param startDate is the first date to be printed
	 * @param endDate is the last date to be printed
	 */
	KDPPrintJobFull( QPrinter* printer, CalObject* calendar, QDate startDate,
		QDate endDate );
	virtual ~KDPPrintJobFull( )
	{}

	/**
	 *
	 * Initialises the print job, by determining how many pages it will take
	 *  to print the desired view.
	 *
	 * @return returns false if we run into problems, true if everything's OK
	 */
	virtual bool init( ) = 0;

	/**
	 *
	 * Prints the data constructed in init()
	 *
	 * @return returns false if we run into problems, true if everything's OK
	 * @param parent is the top-level widget, so we can construct a progress bar
	 */
	virtual bool print( QWidget* parent ) = 0;

protected:

	/**
	 *
	 * Return the printable width of the page in 72dpi units, assuming 
	 *  one inch margins.
	 *
	 * @return printable width in 72dpi units
	 * @param  QPrinter::PageSize
	 */
	int getPrintableWidth( QPrinter::PageSize pagesize );
	/**
	 *
	 * Return the printable height of the page in 72dpi units, assuming 
	 *  one inch margins.
	 *
	 * @return printable height in 72dpi units
	 * @param  QPrinter::PageSize
	 */
	int getPrintableHeight( QPrinter::PageSize pagesize );

	// last page to be printed
	int lastPage;
	// offset down that last page
	int lastOffset;

};

/**
 *
 * This class prints a series of list views on full printer pages.
 *
 * @short prints list views on full printer pages
 * @author Neil Hart
 * @version $Revision: 1.1 $
 */
class KDPPrintJobFullList : public KDPPrintJobFull
{
public:
  /**
   *
   * Constructs a new print job
   *
	 * @param printer is the QPrinter we'll print on
	 * @param calendar is the CalObject from which we'll get event info
	 * @param startDate is the first date to be printed
	 * @param endDate is the last date to be printed
	 */
	KDPPrintJobFullList( QPrinter* printer, CalObject* calendar,
		QDate startDate, QDate endDate );
	virtual ~KDPPrintJobFullList( );
	virtual bool init( );
	virtual bool print( QWidget* parent );

protected:

	// list of KDPPrintObjectList's, one for each day to be printed
	QList<KDPPrintObjectList> *poList;

};

/**
 *
 * This is the abstract top-level class of classes which print their data
 *  two to a printer page.
 *
 * @short prints data two to a printer page
 * @author Neil Hart
 * @version $Revision: 1.1 $
 */
class KDPPrintJobHalf : public KDPPrintJob
{
public:
	/**
	 *
	 * Constructs a new print job
	 *
	 * @param printer is the QPrinter we're printing on
	 * @param calendar is the CalObject from which we'll get event info
	 * @param startDate is the first date to be printed
	 * @param endDate is the last date to be printed
	 */
	KDPPrintJobHalf( QPrinter* printer, CalObject* calendar, QDate startDate,
		QDate endDate );
	virtual ~KDPPrintJobHalf( )
	{}

	/**
	 *
	 * Initialises the print job, by determining how many pages it will take
	 *  to print the desired view.
	 *
	 * @return returns false if we run into problems, true if everything's OK
	 */
	virtual bool init( ) = 0;

	/**
	 *
	 * Prints the data constructed in init()
	 *
	 * @return returns false if we run into problems, true if everything's OK
	 * @param parent is the top-level widget, so we can construct a progress bar
	 */
	virtual bool print( QWidget* parent ) = 0;

protected:

	/**
	 *
	 * Return the printable width of the page in 72dpi units, assuming 
	 *  one inch margins. Note that because we're printing two to a page,
	 *  we're actually printing landscape, so the printable width is half
	 *  the long dimension of a page, minus the margins.
	 *
	 * @return printable width in 72dpi units
	 * @param  QPrinter::PageSize
	 */
	int getPrintableWidth( QPrinter::PageSize pagesize );
	/**
	 *
	 * Return the printable height of the page in 72dpi units, assuming 
	 *  one inch margins.
	 *
	 * @return printable height in 72dpi units
	 * @param  QPrinter::PageSize
	 */
	int getPrintableHeight( QPrinter::PageSize pagesize );

	// last page to be printed
	int lastPage;
	// offset down that last page
	int lastOffset;

};

/**
 *
 * This class prints a series of list views, two to a printer page.
 *
 * @short prints list views, two to a printer page
 * @author Neil Hart
 * @version $Revision: 1.1 $
 */
class KDPPrintJobHalfList : public KDPPrintJobHalf
{
public:
  /**
   *
   * Constructs a new print job
   *
	 * @param printer is the QPrinter we'll print on
	 * @param calendar is the CalObject from which we'll get event info
	 * @param startDate is the first date to be printed
	 * @param endDate is the last date to be printed
	 */
	KDPPrintJobHalfList( QPrinter* printer, CalObject* calendar,
		QDate startDate, QDate endDate );
	virtual ~KDPPrintJobHalfList( );
	virtual bool init( );
	virtual bool print( QWidget* parent );

protected:

	// list of KDPPrintObjectList's, one for each day to be printed
	QList<KDPPrintObjectList> *poList;

};
#endif // _PRINTJOB_H

