/*

   	 $Id: printobject.cpp,v 1.1 1998/07/05 22:41:49 boloni Exp $	

	 Requires the Qt and KDE widget libraries, available at no cost at
	 http://www.troll.no and http://www.kde.org respectively
	 
	 Copyright (C) 1997, 1998 
	 Preston Brown (preston.brown@yale.edu)
	 Fester Zigterman (F.J.F.ZigtermanRustenburg@student.utwente.nl)
	 Ian Dawes (iadawes@globalserve.net)
	 Neil Hart (nhart@magma.ca)
	 
	 This program is free software; you can redistribute it and/or modify
	 it under the terms of the GNU General Public License as published by
	 the Free Software Foundation; either version 2 of the License, or
	 (at your option) any later version.
	 
	 This program is distributed in the hope that it will be useful,
	 but WITHOUT ANY WARRANTY; without even the implied warranty of
	 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
	 GNU General Public License for more details.
	 
	 You should have received a copy of the GNU General Public License
	 along with this program; if not, write to the Free Software
	 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "printobject.h"

KDPPrintObject::KDPPrintObject( QPainter* painter, CalObject* calendar,
	QDate date )
{
	// set up all the pointers
	// make line list do deep copies
	lineList = new QStrList( TRUE );
	currEvents = (KDPEventList *)calendar->getSortedEventsForDate( date );
	thisDate = date;
	prtPainter = painter;
}

KDPPrintObject::~KDPPrintObject( )
{
	delete lineList;
}

KDPPrintObjectList::KDPPrintObjectList( QPainter* painter, CalObject* calendar,
	QDate date ) : KDPPrintObject( painter, calendar, date )
{
}

KDPPrintObjectList::~KDPPrintObjectList( )
{
}

void writeTimeAmPm( int* hour, QString* ampm );
void appendToLineList( QStrList* lineList, QString& str, int* pageNum,
	int* pageOffset, int pageWidth, int height, QFontMetrics& metrics, 
	QString& prefix);

bool KDPPrintObjectList::init( int width, int height, int* pageNum,
	int* pageOffset )
{
	// keep track of the page dimensions --
	// they'll come in handy when we're printing
	pageHeight = height;
	pageWidth  = width;

	// create some prefixes for the lines
	QString noPrefix("");
	QString oneTab("    ");

	// need a QFontMetrics to keep track of how much space we're using
	QFontMetrics metrics = prtPainter->fontMetrics();

	// make local copies of starting page number and offset
	firstPage = *pageNum;
	startOffset = *pageOffset;

	// variables to store running indication of pageNum and pageOffset 
	int currentPageNum = firstPage;
	int currentPageOffset = startOffset;

	// Ensure offset and page number make sense
	if ( ( currentPageOffset + metrics.height() ) >= pageHeight )
	{
		firstPage = currentPageNum = firstPage + 1;
		startOffset = currentPageOffset = 0;
	}
	// Allow space for a dividing line and printing the date
	currentPageOffset += (3 + metrics.height());

	// cycle through the events, adding each to the lineList. We 
	//  assume that the event list is already sorted by time.
	// This code adapted from ListView::makeEntries
	KDPEvent *anEvent;
	QTime t1, t2;
	QString str, str1, str2, ampm1, ampm2;
	int h1, h2;

	for (anEvent = currEvents->first(); anEvent; anEvent = currEvents->next())
	{
		str = "";
		// Don't know quite what to make of this stuff, so I'll comment 
		// it out for now
		/*
		if (anEvent->getAlarmRepeatCount())
			str += "B";
		else
			str += " \n";
		if (anEvent->doesRecur())
			str += "R\n";
		else
			str += " \n";
		if (anEvent->doesFloat())
		{
			str += " \n" + anEvent->getSummary();
		}
		else
		*/
		{
			t1 = anEvent->getDtStart().time();
			t2 = anEvent->getDtEnd().time();

			h1 = t1.hour();
			writeTimeAmPm( &h1, &ampm1 );
			h2 = t2.hour();
			writeTimeAmPm( &h2, &ampm2 );

			str2.sprintf("%d:%02d %s-%d:%02d %s %s",
				h1, t1.minute(), ampm1.data(),
				h2, t2.minute(), ampm2.data(),
				anEvent->getSummary().data());
			str += str2;
		}
		// now we have a line with the start and end time, and a short
		// summary. We'll print this, and then print the longer description
		// below it.
		appendToLineList( lineList, str, &currentPageNum, &currentPageOffset,
			width, pageHeight, metrics, noPrefix);
		str.sprintf("%s", anEvent->getDescription().data());
		appendToLineList( lineList, str, &currentPageNum, &currentPageOffset,
			width, pageHeight, metrics, oneTab);
	}


	// now we've inserted all the entries, and currentPageNum and 
	//  currentPageOffset indicate how much space we've used
	lastPage = currentPageNum;
	*pageNum = currentPageNum;
	*pageOffset = currentPageOffset;

	return true;
}

// Return length of the segment of the string, starting at startPos, which
//  we can print within a maximum width of lineWidth. Return
// -1 if there's nothing more to print (ie. startPos = length of the string)
int findBreakingLen( QString& str, int startPos, int lineWidth,
	QFontMetrics& metrics );

// Append the data to lineList, updating pageNum and pageOffset as needed.
// If the line is too long, do some messing around to break it up and add
// it as separate lines. Each line is prefixed by the specified prefix --
// this allows the summary lines to be set off from the following descriptions
void appendToLineList( QStrList* lineList, QString& str, int* pageNum,
	int* pageOffset, int pageWidth, int height, QFontMetrics& metrics,
	QString& prefix)
{
	int fontHeight = metrics.height();
	int widthOfPrefix = metrics.width( prefix.data() );

	int currWidth = metrics.width( str.data() );
	if ( (currWidth + widthOfPrefix) < pageWidth )
	{
		// it's shorter than a line width, so just add it as is
		lineList->append( QString( prefix + str ).data());
		if ( *pageOffset >= height )
		{
			(*pageNum)++;
			*pageOffset = 0;
		}
		*pageOffset += fontHeight;
	}
	else
	{
		int startPos = 0;
		int breakLen = findBreakingLen( str, startPos,
			( pageWidth - widthOfPrefix), metrics );
		while ( breakLen != -1 )
		{
			lineList->append(
				QString( prefix + str.mid( startPos, breakLen )).data() );
			if ( *pageOffset >= height )
			{
				(*pageNum)++;
				*pageOffset = 0;
			}
			*pageOffset += fontHeight;
			startPos += breakLen;
			breakLen = findBreakingLen( str, startPos,
				( pageWidth - widthOfPrefix ), metrics );
		}	
	}
}

int findBreakingLen( QString& str, int startPos, int lineWidth,
	QFontMetrics& metrics )
{
	// how many characters do we need to print?
	int stringLen = str.length() - startPos + 1;
	// have we already printed the whole string?
	if ( stringLen <= 0 )
		return -1;

	// how much space will the string take if we don't break it?
	QString currSegment( str.mid( startPos, stringLen ) );
	int currWidth = metrics.width( currSegment.data() );

	while ( currWidth > lineWidth )
	{
		// find the next space before our current position
		int breakPos = str.findRev( ' ', (startPos + stringLen - 1) );
		if ( (breakPos == -1) || (breakPos <= startPos) )
			// we can't find a space, so just return the full string length
			break;

		// get the segment of the string which we're proposing to print
		stringLen = breakPos - startPos;
		currSegment = QString( str.mid( startPos, stringLen ) );
		// how much space will it take up?
		currWidth =  metrics.width( currSegment.data() );
	}
	return stringLen;
}

bool KDPPrintObjectList::print( int page )
{
	// do we need to print anything on this page?
	if ( ( page < firstPage ) || ( page > lastPage ) )
		return true;

	// need a QFontMetrics to keep track of how much space we're using
	QFontMetrics metrics = prtPainter->fontMetrics();
	int fontHeight = metrics.height();
	printf(" pageWidth is %d\n", pageWidth );

	// variables to store running indication of page number and offset
	int currentPageNum = firstPage;
	int currentPageOffset = startOffset;

	// print dividing line and our date
	// note that this is complicated by the fact that this version of
	// drawText treats its y coordinate as the base line (ie. the 
	// bottom of the characters)
	currentPageOffset += 3;
	prtPainter->drawLine( 0, ( currentPageOffset - fontHeight ),
		pageWidth, ( currentPageOffset - fontHeight ) );
	QString dateString = thisDate.toString();
	QFont currentFont = prtPainter->font();
	QFont boldFont( currentFont );
	boldFont.setBold( TRUE );
	prtPainter->setFont( boldFont );
	prtPainter->drawText( 0, currentPageOffset, dateString.data() );
	prtPainter->setFont( currentFont );
	currentPageOffset += fontHeight;

	// now, cycle through the lineList, printing each in turn
	char* line;
	for ( line = lineList->first(); line != 0; line = lineList->next() )
	{
		if ( currentPageNum == page )
			prtPainter->drawText( 0, currentPageOffset, line );
		currentPageOffset += fontHeight;
		if ( currentPageOffset >= pageHeight )
		{
			currentPageNum++;
			currentPageOffset = 0;
		}
		if ( currentPageNum > page )
			// no more lines to print on this page
			break;
	}

	return true;
}

void writeTimeAmPm( int* hour, QString* ampm )
{
	if ( *hour == 0 )
	{
		*hour = 12;
		*ampm = "am";
	}
	else if ( *hour > 11 )
	{
		*ampm = "pm";
		if ( *hour != 12 )
		{
			*hour -= 12;
		} 
	}
	else
	{
		*ampm = "am";
	}
}
